from swarms.structs.autoscaler import AutoScaler
from swarms.swarms.god_mode import GodMode
from swarms.swarms.multi_agent_collab import MultiAgentCollaboration
from swarms.swarms.base import AbstractSwarm

__all__ = [
    "AutoScaler",
    "GodMode",
    "MultiAgentCollaboration",
    "AbstractSwarm",
]



from swarms.structs.agent import Agent
from swarms.structs.sequential_workflow import SequentialWorkflow
from swarms.structs.autoscaler import AutoScaler
from swarms.structs.conversation import Conversation

__all__ = ["Agent", "SequentialWorkflow", "AutoScaler", "Conversation"]



import re

# def extract_code_in_backticks_in_string(s: str) -> str:
# """
# Extracts code blocks from a markdown string.

# Args:
#     s (str): The markdown string to extract code from.

# Returns:
#     list: A list of tuples. Each tuple contains the language of the code block (if specified) and the code itself.
# """
# pattern = r"```([\w\+\#\-\.\s]*)\n(.*?)```"
# matches = re.findall(pattern, s, re.DOTALL)
# out =  [(match[0], match[1].strip()) for match in matches]
# print(out)


def extract_code_in_backticks_in_string(s: str) -> str:
    """
    Extracts code blocks from a markdown string.

    Args:
        s (str): The markdown string to extract code from.

    Returns:
        str: A string containing all the code blocks.
    """
    pattern = r"```([\w\+\#\-\.\s]*)(.*?)```"
    matches = re.findall(pattern, s, re.DOTALL)
    return "\n".join(match[1].strip() for match in matches)



from dataclasses import dataclass, field
from typing import (
    Any,
    Callable,
    Dict,
    List,
    Union,
)

from swarms.structs.agent import Agent


# Define a generic Task that can handle different types of callable objects
@dataclass
class Task:
    """
    Task class for running a task in a sequential workflow.


    Args:
        description (str): The description of the task.
        agent (Union[Callable, Agent]): The model or agent to execute the task.
        args (List[Any]): Additional arguments to pass to the task execution.
        kwargs (Dict[str, Any]): Additional keyword arguments to pass to the task execution.
        result (Any): The result of the task execution.
        history (List[Any]): The history of the task execution.

    Methods:
        execute: Execute the task.


    Examples:
    >>> from swarms.structs import Task, Agent
    >>> from swarms.models import OpenAIChat
    >>> agent = Agent(llm=OpenAIChat(openai_api_key=""), max_loops=1, dashboard=False)
    >>> task = Task(description="What's the weather in miami", agent=agent)
    >>> task.execute()
    >>> task.result

    """

    description: str
    agent: Union[Callable, Agent]
    args: List[Any] = field(default_factory=list)
    kwargs: Dict[str, Any] = field(default_factory=dict)
    result: Any = None
    history: List[Any] = field(default_factory=list)
    # logger = logging.getLogger(__name__)

    def execute(self):
        """
        Execute the task.

        Raises:
            ValueError: If a Agent instance is used as a task and the 'task' argument is not provided.
        """
        if isinstance(self.agent, Agent):
            # Add a prompt to notify the Agent of the sequential workflow
            if "prompt" in self.kwargs:
                self.kwargs["prompt"] += (
                    f"\n\nPrevious output: {self.result}"
                    if self.result
                    else ""
                )
            else:
                self.kwargs["prompt"] = (
                    f"Main task: {self.description}"
                    + (
                        f"\n\nPrevious output: {self.result}"
                        if self.result
                        else ""
                    )
                )
            self.result = self.agent.run(*self.args, **self.kwargs)
        else:
            self.result = self.agent(*self.args, **self.kwargs)

        self.history.append(self.result)



from swarms.memory.base_vectordb import VectorDatabase
from swarms.memory.short_term_memory import ShortTermMemory
from swarms.memory.sqlite import SQLiteDB

__all__ = ["VectorDatabase", "ShortTermMemory", "SQLiteDB"]



from swarms.models.base_llm import AbstractLLM


try:
    import multion

except ImportError:
    raise ImportError(
        "Cannot import multion, please install 'pip install'"
    )


class MultiOn(AbstractLLM):
    """
    MultiOn is a wrapper for the Multion API.

    Args:
        **kwargs:

    Methods:
        run(self, task: str, url: str, *args, **kwargs)

    Example:
    >>> from swarms.models.multion import MultiOn
    >>> multion = MultiOn()
    >>> multion.run("Order chicken tendies", "https://www.google.com/")
    "Order chicken tendies. https://www.google.com/"

    """

    def __init__(self, **kwargs):
        super(MultiOn, self).__init__(**kwargs)

    def run(self, task: str, url: str, *args, **kwargs) -> str:
        """Run the multion model

        Args:
            task (str): _description_
            url (str): _description_

        Returns:
            str: _description_
        """
        response = multion.new_session({"input": task, "url": url})
        return response

    def generate_summary(
        self, task: str, url: str, *args, **kwargs
    ) -> str:
        """Generate a summary from the multion model

        Args:
            task (str): _description_
            url (str): _description_

        Returns:
            str: _description_
        """
        response = multion.new_session({"input": task, "url": url})
        return response



from swarms.utils.code_interpreter import SubprocessCodeInterpreter
from swarms.utils.markdown_message import display_markdown_message
from swarms.utils.parse_code import (
    extract_code_in_backticks_in_string,
)
from swarms.utils.pdf_to_text import pdf_to_text
from swarms.utils.math_eval import math_eval

# from swarms.utils.phoenix_handler import phoenix_trace_decorator

__all__ = [
    "display_markdown_message",
    "SubprocessCodeInterpreter",
    "extract_code_in_backticks_in_string",
    "pdf_to_text",
    # "phoenix_trace_decorator",
    "math_eval",
]
